import { Injectable } from '@nestjs/common';
import { PartyRepository } from './store/party-repository';
import { Party, PartyStatus } from '@prisma/client';
import { RunService } from '../run/run.service';
import { UserService } from '../user/user.service';
import { AnswerService } from '../answer/answer.service';

@Injectable()
export class PartyService {
  constructor(
    private readonly partyRepository: PartyRepository,
    private readonly runService: RunService,
    private readonly userService: UserService,
    private readonly answerService: AnswerService,
  ) {}
  async findParty(partyId: string): Promise<Party> {
    return await this.partyRepository.findParty(partyId);
  }

  async isPartyOpen(partyId: string): Promise<boolean> {
    return (await this.findParty(partyId)).status === 'open';
  }

  async joinParty(infos: { userId: number; partyId: string }) {
    return await this.partyRepository.joinParty(infos);
  }

  async leaveParty(infos: { userId: number }) {
    return await this.partyRepository.leaveParty(infos.userId);
  }

  async createParty(infos: { hostId: number; quizId: string }) {
    const partyId = await this.generateUniqueId();
    return await this.partyRepository.createParty({
      ...infos,
      partyId,
    });
  }

  async generateRun({ userId }: { userId: number }) {
    const user = await this.userService.findOneWithPartyInfos(userId);

    const quizId = await this.getQuizId(user.Party.id);

    const runId = await this.runService.generateUniqueId();

    const run = await this.runService.create(
      runId,
      quizId,
      quizId,
      user.id,
      user.Party.id,
    );

    return { runId: run.id };
  }

  async generateUniqueId(): Promise<string> {
    const generateCode = () => {
      const characters = 'ABCDEFGHJKLMNPQRSTUVWXYZ0123456789';
      let code = '';
      for (let i = 0; i < 6; i++) {
        code += characters.charAt(
          Math.floor(Math.random() * characters.length),
        );
      }
      return code;
    };

    let id;
    let isUnique = false;

    while (!isUnique) {
      id = generateCode();
      const existingQuiz = await this.partyRepository.findParty(id);
      if (!existingQuiz) {
        isUnique = true;
      }
    }

    return id;
  }

  async setPartyStatus(partyId: string, status: PartyStatus) {
    return await this.partyRepository.setPartyStatus(partyId, status);
  }

  async getQuizId(partyId: string) {
    return await this.partyRepository.getQuizId(partyId);
  }

  async createRun(userId: number, partyId: string) {
    const quizId = await this.getQuizId(partyId);

    const runId = await this.runService.generateUniqueId();

    const run = await this.runService.create(
      runId,
      quizId,
      quizId,
      userId,
      partyId,
    );

    return { run: run };
  }

  async getQuestion({ id }: { id: string }) {
    try {
      const question = await this.runService.getQuestions(id);

      if (!question) {
        throw new Error(
          `No more questions available for run ${id}. The quiz has beeen completed.`,
        );
      }

      return question;
    } catch (error) {
      if (
        error.message.includes('No more questions available') ||
        error.message.includes('completed')
      ) {
        throw new Error(error.message);
      }

      throw new Error(
        'Failed to retrieve the question. Please try again later.',
      );
    }
  }

  async answer({
    runId,
    questionId,
    answerId,
  }: {
    runId: string;
    questionId: number;
    answerId: number;
  }) {
    const isCorrect = await this.answerService.isRunCorrect(
      runId,
      questionId,
      answerId,
    );

    if (isCorrect) {
      await this.runService.incrementScore(runId);
    }

    await this.runService.incrementQuestionIndex(runId);
  }

  async getScoreboard(partyId: string) {
    return await this.partyRepository.getScores(partyId);
  }
}
