import {
  Controller,
  Get,
  Post,
  Body,
  Patch,
  Param,
  Delete,
  UseGuards,
  Req,
  Query,
  HttpException,
  HttpStatus,
} from '@nestjs/common';
import { QuizService } from './quiz.service';
import { CreateQuizDto } from './dto/create-quiz.dto';
import { UpdateQuizDto } from './dto/update-quiz.dto';
import { AuthGuard } from '../../Auth/auth/auth.guard';
import { UserGuard } from '../user/user.guard';
import { CreateCommunityQuizDto } from './dto/community-quiz.dto';
import { OptionalAuthGuard } from '../../Auth/optionalAuth/OptionalAuthGuard';

@Controller('quiz')
export class QuizController {
  constructor(private readonly quizService: QuizService) {}

  @Post()
  async create(@Body() createQuizDto: CreateQuizDto) {
    try {
      return await this.quizService.create(createQuizDto);
    } catch (error) {
      console.error('Error in create:', error);
      throw new HttpException('Failed to create quiz', HttpStatus.BAD_REQUEST);
    }
  }

  @UseGuards(OptionalAuthGuard)
  @Post('generate')
  async generateQuiz(
    @Body()
    body: {
      amount: number;
      category: number;
      difficulty: string;
    },
    @Req() req: any,
  ) {
    try {
      const userId = req.userId; // Récupère l'userId ou le met à null s'il est absent
      const { amount, category, difficulty } = body;
      return await this.quizService.importQuestions(
        amount,
        category,
        difficulty,
        userId,
      );
    } catch (error) {
      console.error('Error in generateQuiz:', error);
      throw new HttpException(
        'Failed to generate quiz',
        HttpStatus.BAD_REQUEST,
      );
    }
  }

  @UseGuards(AuthGuard, UserGuard)
  @Post('community')
  async community(
    @Body() createCommunityQuizDto: CreateCommunityQuizDto,
    @Req() req: any,
  ) {
    const authorId = req.userId;
    try {
      const quiz = await this.quizService.createCommunityQuiz(
        createCommunityQuizDto,
        authorId,
      );
      return { id: quiz.id };
    } catch (error) {
      console.error('Error in community:', error);
      throw new HttpException(
        'Failed to create community quiz',
        HttpStatus.BAD_REQUEST,
      );
    }
  }

  @Get('categories')
  async categories() {
    try {
      return await this.quizService.getCategories();
    } catch (e) {
      console.error('Error in getCategories:', e);
      throw new HttpException(
        'Failed to get categories',
        HttpStatus.BAD_REQUEST,
      );
    }
  }

  @Get('info/:id')
  async info(@Param('id') id: string) {
    try {
      return await this.quizService.info(id);
    } catch (error) {
      console.error('Error in info:', error);
      throw new HttpException(
        'Failed to get quiz info',
        HttpStatus.BAD_REQUEST,
      );
    }
  }

  @Post('stats/:id')
  async getStats(@Param('id') id: string) {
    try {
      return await this.quizService.getStats(id);
    } catch (error) {
      console.error('Error in getStats:', error);
      throw new HttpException('Failed to get stats', HttpStatus.BAD_REQUEST);
    }
  }

  @Get('all')
  async findAll() {
    try {
      return await this.quizService.findAll();
    } catch (error) {
      console.error('Error in findAll:', error);
      throw new HttpException(
        'Failed to retrieve quizzes',
        HttpStatus.BAD_REQUEST,
      );
    }
  }

  @Patch(':id')
  async update(@Param('id') id: string, @Body() updateQuizDto: UpdateQuizDto) {
    try {
      return await this.quizService.update(+id, updateQuizDto);
    } catch (error) {
      console.error('Error in update:', error);
      throw new HttpException('Failed to update quiz', HttpStatus.BAD_REQUEST);
    }
  }

  @Delete(':id')
  async remove(@Param('id') id: string) {
    try {
      return await this.quizService.remove(+id);
    } catch (error) {
      console.error('Error in remove:', error);
      throw new HttpException('Failed to remove quiz', HttpStatus.BAD_REQUEST);
    }
  }

  @Get('paginated')
  async getPaginatedQuizzes(
    @Query('skip') skip: number,
    @Query('take') take: number,
    @Query('search') search: string,
  ) {
    try {
      return await this.quizService.getPaginatedQuizzes(
        Number(skip),
        Number(take),
        search,
      );
    } catch (error) {
      console.error('Error in getPaginatedQuizzes:', error);
      throw new HttpException(
        'Failed to retrieve paginated quizzes',
        HttpStatus.BAD_REQUEST,
      );
    }
  }

  @Get(':id')
  async findOne(@Param('id') id: string) {
    try {
      return await this.quizService.findOne(id);
    } catch (error) {
      console.error('Error in findOne:', error);
      throw new HttpException(
        'Failed to retrieve quiz',
        HttpStatus.BAD_REQUEST,
      );
    }
  }

  @Get('user/:id')
  async getUserCustomQuiz(@Param('id') id: string) {
    try {
      return await this.quizService.getUserCustomQuiz(id);
    } catch (error) {
      console.error('Error in getPaginatedQuizzes:', error);
      throw new HttpException(
        'Failed to retrieve paginated quizzes',
        HttpStatus.BAD_REQUEST,
      );
    }
  }
}
