import {
  Controller,
  Get,
  Post,
  Body,
  Patch,
  Param,
  Delete,
  UseGuards,
  Req,
  HttpStatus,
  HttpException,
  Inject,
  forwardRef,
} from '@nestjs/common';
import { RunService } from './run.service';
import { UpdateRunDto } from './dto/update-run.dto';
import { AuthGuard } from '../../Auth/auth/auth.guard';
import { UserGuard } from '../user/user.guard';
import { UserRequest } from 'src/utils/UserRequest';
import { AnswerService } from '../answer/answer.service';
import { OptionalAuthGuard } from '../../Auth/optionalAuth/OptionalAuthGuard';
import { QuizService } from '../quiz/quiz.service';
import { UserService } from '../user/user.service';

@Controller('run')
export class RunController {
  constructor(
    private readonly runService: RunService,
    private readonly answerService: AnswerService,
    @Inject(forwardRef(() => QuizService))
    private readonly quizService: QuizService,
    private readonly userService: UserService,
  ) {}

  @Get()
  findAll() {
    return this.runService.findAll();
  }

  @Post('answer')
  async answer(
    @Body() body: { id: string; questionID: number; answerID: number },
  ) {
    const { id, questionID, answerID } = body;

    const isCorrect = await this.answerService.isRunCorrect(
      id,
      questionID,
      answerID,
    );

    if (isCorrect) {
      await this.runService.incrementScore(id);
    }

    await this.runService.incrementQuestionIndex(id);

    const score = await this.runService.getScore(id);

    const answers = await this.runService.getAnswers(id, questionID);

    return { score, answers };
  }

  @Post('question')
  async getQuestion(@Body() body: { id: string }) {
    try {
      const { id } = body;

      // Récupérer la question actuelle
      const question = await this.runService.getQuestions(id);

      if (!question) {
        throw new Error(
          `No more questions available for run ID ${id}. The quiz has been completed.`,
        );
      }

      return { question };
    } catch (error) {
      console.error(`Error in getQuestion: ${error.message}`);

      if (
        error.message.includes('No more questions available') ||
        error.message.includes('completed')
      ) {
        throw new HttpException(error.message, HttpStatus.NOT_FOUND);
      }

      throw new HttpException(
        'Failed to retrieve the question. Please try again later.',
        HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }

  @UseGuards(OptionalAuthGuard)
  @Post('start')
  async start(@Body() body, @Req() req: any) {
    const { quizId } = body;
    const userId = req.userId;
    const run = await this.quizService.startRun(quizId, userId);
    return { id: run.id };
  }

  @UseGuards(AuthGuard, UserGuard)
  @Get('userRuns')
  async info(@Req() req: UserRequest) {
    const userId = req.userId;
    const runs = await this.runService.getRunsByUser(userId);
    return { runs };
  }

  @Get('info/:id')
  async getInfos(@Param('id') id: string) {
    return this.runService.findOne(id);
  }

  @Post('remaining')
  async remaining(@Body() body: { id: string }) {
    const { id } = body;
    const remaining = await this.runService.findRemainingQuestions(id);
    return { remaining };
  }

  @Patch(':id')
  update(@Param('id') id: string, @Body() updateRunDto: UpdateRunDto) {
    return this.runService.update(id, updateRunDto);
  }

  @Delete(':id')
  remove(@Param('id') id: string) {
    return this.runService.remove(id);
  }
}
