import { Test, TestingModule } from '@nestjs/testing';
import { UserController } from './user.controller';
import { UserService } from './user.service';
import { CreateUserDto } from './dto/create-user.dto';
import { UpdateUserDto } from './dto/update-user.dto';
import { JwtService } from '@nestjs/jwt';
import {
  BadRequestException,
  InternalServerErrorException,
} from '@nestjs/common';

describe('UserController', () => {
  let controller: UserController;
  let userService: UserService;

  beforeEach(async () => {
    const module: TestingModule = await Test.createTestingModule({
      controllers: [UserController],
      providers: [
        {
          provide: UserService,
          useValue: {
            create: jest.fn(),
            findAll: jest.fn(),
            findOne: jest.fn(),
            update: jest.fn(),
            remove: jest.fn(),
          },
        },
        {
          provide: JwtService,
          useValue: {
            sign: jest.fn(),
            verify: jest.fn(),
          },
        },
      ],
    }).compile();

    controller = module.get<UserController>(UserController);
    userService = module.get<UserService>(UserService);
  });

  afterEach(() => {
    jest.clearAllMocks();
  });

  it('should be defined', () => {
    expect(controller).toBeDefined();
  });

  describe('create', () => {
    it('should create a user', async () => {
      const createUserDto: CreateUserDto = {
        username: 'John Doe',
        email: 'john@example.com',
        password: 'password',
      };
      const result = { id: 1, ...createUserDto };
      jest.spyOn(userService, 'create').mockImplementation(async () => result);

      expect(await controller.create(createUserDto)).toBe(result);
    });

    it('should throw an error if the user is not created', async () => {
      const createUserDto: CreateUserDto = {
        username: 'John Doe',
        email: 'john@example.com',
        password: 'password',
      };

      jest.spyOn(userService, 'create').mockImplementation(async () => null);

      try {
        await controller.create(createUserDto);
      } catch (error) {
        expect(error.message).toBe('Username already taken');
      }
    });

    it('should throw an error if the username is already taken', async () => {
      const createUserDto: CreateUserDto = {
        username: 'John Doe',
        email: 'john@example.com',
        password: 'password',
      };

      jest.spyOn(userService, 'create').mockImplementation(async () => {
        throw new BadRequestException('Username already taken');
      });

      try {
        await controller.create(createUserDto);
      } catch (error) {
        expect(error.status).toBe(400);
        expect(error.message).toBe('Username already taken');
      }
    });
  });

  describe('find', () => {
    it('should return an array of users', async () => {
      const result = [
        { id: 1, username: 'John Doe', email: 'john@example.com' },
      ];
      jest.spyOn(userService, 'findAll').mockImplementation(async () => result);

      expect(await controller.findAll()).toBe(result);
    });

    it('should return a user by id', async () => {
      const result = {
        id: 1,
        username: 'John Doe',
        email: 'john@example.com',
        stats: {
          quizzesCreated: 2,
          quizzesPlayed: 1,
          correctAnswers: 0,
          totalAnswers: 0,
        },
      };
      jest.spyOn(userService, 'findOne').mockImplementation(async () => result);

      expect(await controller.findOne('1')).toBe(result);
    });
  });

  describe('update', () => {
    it('should update a user', async () => {
      const updateUserDto: UpdateUserDto = { id: 1, username: 'Jane Doe' };
      const result = {
        id: 1,
        username: 'Jane Doe',
        email: 'jane@example.com',
        password: 'password',
      };
      jest.spyOn(userService, 'update').mockImplementation(async () => result);

      expect(await controller.update('1', updateUserDto)).toBe(result);
    });

    it('should throw an error if the user is not found', async () => {
      const updateUserDto: UpdateUserDto = { id: 1, username: 'Jane Doe' };

      jest.spyOn(userService, 'update').mockImplementation(async () => null);

      try {
        await controller.update('1', updateUserDto);
      } catch (error) {
        expect(error.status).toBe(404);
      }
    });

    it('should throw an error if the user is not updated', async () => {
      const updateUserDto: UpdateUserDto = { id: 1, username: 'Jane Doe' };

      jest.spyOn(userService, 'update').mockImplementation(async () => {
        throw new InternalServerErrorException('Error updating user');
      });

      try {
        await controller.update('1', updateUserDto);
      } catch (error) {
        expect(error.status).toBe(500);
        expect(error.message).toBe('Error updating user');
      }
    });
  });

  describe('remove', () => {
    it('should remove a user', async () => {
      const result = { id: 1, username: 'John Doe', email: 'john@example.com' };
      jest.spyOn(userService, 'remove').mockImplementation(async () => result);

      expect(await controller.remove('1')).toBe(result);
    });

    it('should throw an error if the user is not found', async () => {
      jest.spyOn(userService, 'remove').mockImplementation(async () => null);

      try {
        await controller.remove('1');
      } catch (error) {
        expect(error.status).toBe(404);
      }
    });
  });
});
