import { Test, TestingModule } from '@nestjs/testing';
import { UserService } from './user.service';

describe('UserService', () => {
  let userService: UserService;

  beforeEach(async () => {
    const module: TestingModule = await Test.createTestingModule({
      providers: [
        {
          provide: UserService,
          useValue: {
            findOne: jest.fn(),
            findOneByUsername: jest.fn(),
            findOneByLogin: jest.fn(),
            update: jest.fn(),
          },
        },
      ],
    }).compile();

    userService = module.get<UserService>(UserService);
  });

  it('should be defined', () => {
    expect(userService).toBeDefined();
  });

  describe('find', () => {
    describe('findOne', () => {
      it('should find a user', async () => {
        const result = {
          id: 1,
          username: 'John Doe',
          email: 'john@example.com',
          stats: {
            quizzesCreated: 0,
            quizzesPlayed: 0,
            correctAnswers: 0,
            totalAnswers: 0,
          },
        };

        jest
          .spyOn(userService, 'findOne')
          .mockImplementation(async () => result);

        expect(await userService.findOne(1)).toBe(result);
      });

      it('should throw an error if the user is not found', async () => {
        jest.spyOn(userService, 'findOne').mockImplementation(async () => null);

        try {
          await userService.update({ id: 1, username: 'Jane Doe' });
        } catch (error) {
          expect(error.status).toBe(404);
        }
      });
    });

    describe('findOneByUsername', () => {
      it('should find a user by username', async () => {
        const result = {
          id: 1,
          username: 'John Doe',
          email: 'john@example.com',
        };

        jest
          .spyOn(userService, 'findOneByUsername')
          .mockImplementation(async () => result);

        expect(await userService.findOneByUsername('John Doe')).toBe(result);
      });

      it('should throw an error if the user is not found', async () => {
        jest
          .spyOn(userService, 'findOneByUsername')
          .mockImplementation(async () => null);

        try {
          await userService.update({ id: 1, username: 'Jane Doe' });
        } catch (error) {
          expect(error.status).toBe(404);
        }
      });
    });

    describe('findOneByLogin', () => {
      it('should find a user by login with username', async () => {
        const result = {
          id: 1,
          username: 'John Doe',
          email: 'john@example.com',
        };

        jest
          .spyOn(userService, 'findOneByLogin')
          .mockImplementation(async () => result);

        expect(await userService.findOneByLogin('John Doe')).toBe(result);
      });

      it('should find a user by login with email', async () => {
        const result = {
          id: 1,
          username: 'John Doe',
          email: 'john@example.com',
        };

        jest
          .spyOn(userService, 'findOneByLogin')
          .mockImplementation(async () => result);

        expect(await userService.findOneByLogin('john@example.com')).toBe(
          result,
        );
      });

      it('should throw an error if the user is not found', async () => {
        jest
          .spyOn(userService, 'findOneByLogin')
          .mockImplementation(async () => null);

        try {
          await userService.update({ id: 1, username: 'Jane Doe' });
        } catch (error) {
          expect(error.status).toBe(404);
        }
      });
    });

    it('should update a user', async () => {
      const result = { id: 1, username: 'John Doe', email: 'john@example.com' };

      jest.spyOn(userService, 'update').mockImplementation(async () => result);

      expect(await userService.update({ id: 1, username: 'Jane Doe' })).toBe(
        result,
      );
    });
  });
});
